/* frame rate measurement
 *
 * (C) 2018 by Andreas Eversberg <jolly@eversberg.eu>
 * All Rights Reserved
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include "../../include/keycodes.h"
#include "../libsdl/sdl.h"
#include "framerate.h"

#define MAX_HISTORY 60

double vbl_duration = 1.0 / 60.0;
static uint32_t ticks_history[MAX_HISTORY];
static int ticks_count = 0;

void framerate_measure(void)
{
	uint32_t ticks = ticks_sdl();

	/* wait some ticks until to get better average when we start */
	if (ticks_count >= 10 || ticks_count == MAX_HISTORY)
		vbl_duration = (double)((uint32_t)(ticks - ticks_history[0])) / (double)ticks_count / 1000.0;

	if (ticks_count == MAX_HISTORY) {
		memmove(ticks_history, ticks_history + 1, sizeof(ticks_history[0]) * (MAX_HISTORY - 1));
		ticks_count--;
	}
	ticks_history[ticks_count++] = ticks;
}

